aprs = {}

-- compose an aprs message body
function aprs.compose_message(callsign, message, id)
  for i = 1, 9 do
    if string.sub(callsign, i, i) == "" then
      callsign = callsign .. " "
    end
  end
  message = string.gsub(message, "[\000-\031\128-\255|~{]", "_")
  local aprs_message = ":" .. callsign .. ":" .. message
  if id then
    aprs_message = aprs_message .. "{" .. id
  end
  return aprs_message
end

-- compose an aprs bulleting body
function aprs.compose_bulletin(destination, message, id)
  for i = 1, 9 do
    if string.sub(destination, i, i) == "" then
      destination = destination .. " "
    end
  end
  message = string.gsub(message, "[\000-\031\128-\255|~{]", "_")
  local aprs_message = ":" .. destination .. ":" .. message
  if id then
    aprs_message = aprs_message .. "{" .. id
  end
  return aprs_message
end

-- compose an aprs user defined chat frame
function aprs.compose_chat(channel_name, message, id)
  --message = string.gsub(message, "[\000-\031\128-\255|~{]", "_")
  channel_name = upper(channel_name)
  local aprs_message = "{{#" .. channel_name .. ":" .. message
  if id then
    aprs_message = aprs_message .. "{" .. id
  end
  return aprs_message
end

-- parse an aprs message body
function aprs.parse_message(aprs_message)
  local station, message, id = string.match(aprs_message, "^:(.........):(.*){([^{][^{]?[^{]?[^{]?[^{]?)$")
  if not id then
    station, message = string.match(aprs_message, "^:(.........):(.*)")
  end
  if not station or not message then
    return
  end
  for i = 9, 1, -1 do
    if string.sub(station, i, i) == " " then
      station = string.sub(station, 1, i-1)
    end
  end
  local callsign, ssid = string.match(station, "^([A-Z0-9]+)-?(.*)$")
  if ssid == "" then 
    ssid = 0
  else
    ssid = tonumber(ssid)
  end
  station = callsign
  if ssid > 0 then
    station = station .. "-" .. ssid
  end
  return {
    message_type = "message",
    station = station,
    callsign = callsign,
    ssid = ssid,
    message = message,
    id = id
  }
end

--[[
-- parse an aprs bulletin body
function aprs.parse_bulletin(aprs_message)
  local destination, message = string.match(aprs_message, "^:(.........):(.*)$")
  for i = 9, 1, -1 do
    if string.sub(destination, i, i) == " " then
      destination = string.sub(destination, 1, i-1)
    end
  end
  return {
    message_type = "bulletin",
    destination = destination,
    message = message,
    id = id
  }
end
--]]

-- parse an aprs user defined chat frame
function aprs.parse_chat(aprs_message)
  local channel_name, message, id = string.match(aprs_message, "^{{#([0-9A-Za-z_]+):(.*){([0-9A-Za-z][0-9A-Za-z][0-9A-Za-z][0-9A-Za-z])$")
  if not id or not channel_name or not message then return end
  local channel_name = string.lower(channel_name)
  return {
    message_type = "chat",
    channel_name = channel_name,
    message = message,
    id = id
  }
end

-- parse an aprs body (bulletin and messages supported)
function aprs.parse(aprs_message)
  if string.sub(aprs_message, 1, 1) == ":" then
    -- APRS message or bulletin
--    if string.match(string.sub(aprs_message, 2, 5), aprs.callsign_detect_regex) then
      return aprs.parse_message(aprs_message)
--    else
--      return aprs.parse_bulletin(aprs_message)
--    end
  elseif string.sub(aprs_message, 1, 3) == "{{#" then
    return aprs.parse_chat(aprs_message)
  end
end

